@@ -1,11 +1,11 @@
-# Copyright 2011 Omniscale (http://omniscale.com)
-# 
+# Copyright 2011-2012 Omniscale (http://omniscale.com)
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -13,6 +13,8 @@
 # limitations under the License.
 
 import time
+import uuid
+from contextlib import contextmanager
 
 import psycopg2
 import psycopg2.extensions
@@ -23,14 +25,26 @@ log = logging.getLogger(__name__)
 from imposm import config
 from imposm.mapping import UnionView, GeneralizedTable, Mapping
 
+unknown = object()
+
 class PostGISDB(object):
-    def __init__(self, db_conf):
+    insert_data_format = 'tuple'
+
+    def __init__(self, db_conf, use_geometry_columns_table=unknown):
         self.db_conf = db_conf
         self.srid = int(db_conf['proj'].split(':')[1])
+
         self._insert_stmts = {}
         self._connection = None
         self._cur = None
 
+        if use_geometry_columns_table is unknown:
+            if self.is_postgis_2():
+                use_geometry_columns_table = False
+            else:
+                use_geometry_columns_table = True
+        self.use_geometry_columns_table = use_geometry_columns_table
+
     @property
     def table_prefix(self):
         return self.db_conf.prefix.rstrip('_') + '_'
@@ -38,6 +52,12 @@ class PostGISDB(object):
     def to_tablename(self, name):
         return self.table_prefix + name.lower()
 
+    def is_postgis_2(self):
+        cur = self.connection.cursor()
+        cur.execute('SELECT postgis_version()')
+        version_string = cur.fetchone()[0]
+        return version_string.strip()[0] == '2'
+
     @property
     def connection(self):
         if not self._connection:
@@ -55,16 +75,27 @@ class PostGISDB(object):
             self._connection.set_isolation_level(
                 psycopg2.extensions.ISOLATION_LEVEL_READ_COMMITTED)
         return self._connection
-    
+
     def commit(self):
         self.connection.commit()
-    
+
     @property
     def cur(self):
         if self._cur is None:
             self._cur = self.connection.cursor()
         return self._cur
-    
+
+    @contextmanager
+    def savepoint(self, cur, raise_errors=False):
+        savepoint_name = 'savepoint' + uuid.uuid4().get_hex()
+        try:
+            cur.execute('SAVEPOINT %s' % savepoint_name)
+            yield
+        except psycopg2.ProgrammingError:
+            cur.execute('ROLLBACK TO SAVEPOINT %s' % savepoint_name)
+            if raise_errors:
+                raise
+
     def insert(self, mapping, insert_data, tries=0):
         insert_stmt = self.insert_stmt(mapping)
         try:
@@ -91,10 +122,24 @@ class PostGISDB(object):
                     self.connection.commit()
 
         self.connection.commit()
-    
+
+    def post_insert(self, mappings):
+        mappings = [m for m in mappings.values() if isinstance(m, (GeneralizedTable, Mapping))]
+        for mapping in mappings:
+            table_name = self.to_tablename(mapping.name)
+            self.create_geom_index(table_name)
+
+    def create_geom_index(self, table_name):
+        idx_name = '%s_geom' % table_name
+        cur = self.connection.cursor()
+        cur.execute("""
+            CREATE INDEX "%s" ON "%s" USING GIST (geometry)
+        """ % (idx_name,table_name))
+        self.connection.commit()
+
     def geom_wrapper(self, geom):
         return psycopg2.Binary(geom.wkb)
-    
+
     def reconnect(self):
         if self._connection:
             try:
@@ -128,14 +173,17 @@ class PostGISDB(object):
         for mapping in mappings:
             self.create_table(mapping)
 
+    def drop_table_or_view(self, cur, name):
+        with self.savepoint(cur):
+            cur.execute('DROP TABLE "' + name + '" CASCADE')
+        with self.savepoint(cur):
+            cur.execute('DROP VIEW "' + name + '" CASCADE')
+
     def create_table(self, mapping):
         tablename = self.table_prefix + mapping.name
         cur = self.connection.cursor()
-        cur.execute('SAVEPOINT pre_drop_tables')
-        try:
-            cur.execute('DROP TABLE "' + tablename + '" CASCADE')
-        except psycopg2.ProgrammingError:
-            cur.execute('ROLLBACK TO SAVEPOINT pre_drop_tables')
+
+        self.drop_table_or_view(cur, tablename)
 
         extra_fields = ''
         for n, t in mapping.fields:
@@ -145,7 +193,7 @@ class PostGISDB(object):
             serial_column = "id SERIAL PRIMARY KEY,"
         else:
             serial_column = ""
-            
+
         cur.execute("""
             CREATE TABLE "%s" (
                 %s
@@ -153,27 +201,42 @@ class PostGISDB(object):
                 %s
             );
         """ % (tablename, serial_column, extra_fields))
-        cur.execute("""
-            SELECT AddGeometryColumn ('', '%(tablename)s', 'geometry',
-                                      %(srid)s, '%(pg_geometry_type)s', 2)
-        """ % dict(tablename=tablename, srid=self.srid,
-                   pg_geometry_type=mapping.geom_type))
 
+        self.create_geometry_column(cur, tablename, mapping)
+
+        self.create_field_indices(cur=cur, mapping=mapping, tablename=tablename)
+
+
+    def create_geometry_column(self, cur, tablename, mapping):
+        if self.use_geometry_columns_table:
+            cur.execute("""
+                SELECT AddGeometryColumn ('', '%(tablename)s', 'geometry',
+                                          %(srid)s, '%(pg_geometry_type)s', 2)
+            """ % dict(tablename=tablename, srid=self.srid,
+                       pg_geometry_type=mapping.geom_type))
+        else:
+            cur.execute("""
+                ALTER TABLE %(tablename)s ADD COLUMN geometry geometry(%(pg_geometry_type)s, %(srid)s);
+            """ % dict(tablename=tablename, srid=self.srid,
+                       pg_geometry_type=mapping.geom_type))
+
+    def create_field_indices(self, cur, mapping, tablename):
         for n, t in mapping.fields:
             if isinstance(t, TrigramIndex):
                 cur.execute("""
                     CREATE INDEX "%(tablename)s_trgm_idx_%(column)s" ON "%(tablename)s" USING GIST ("%(column)s" gist_trgm_ops)
                 """ % dict(tablename=tablename, column=n))
+            if isinstance(t, (StringIndex, Index)):
+                cur.execute("""
+                    CREATE INDEX "%(tablename)s_idx_%(column)s" ON "%(tablename)s" ("%(column)s")
+                """ % dict(tablename=tablename, column=n))
 
-        cur.execute("""
-            CREATE INDEX "%(tablename)s_geom" ON "%(tablename)s" USING GIST (geometry)
-        """ % dict(tablename=tablename))
-    
     def swap_tables(self, new_prefix, existing_prefix, backup_prefix):
         cur = self.connection.cursor()
 
         self.remove_tables(backup_prefix)
-        
+        self.remove_views(backup_prefix)
+
         cur.execute('SELECT tablename FROM pg_tables WHERE tablename like %s', (existing_prefix + '%', ))
         existing_tables = []
         for row in cur:
@@ -182,6 +245,14 @@ class PostGISDB(object):
                 # check for overlapping prefixes: osm_ but not osm_new_ or osm_backup_
                 existing_tables.append(table_name)
 
+        cur.execute('SELECT viewname FROM pg_views WHERE viewname like %s', (existing_prefix + '%', ))
+        existing_views = []
+        for row in cur:
+            view_name = row[0]
+            if view_name.startswith(existing_prefix) and not view_name.startswith((new_prefix, backup_prefix)):
+                # check for overlapping prefixes: osm_ but not osm_new_ or osm_backup_
+                existing_views.append(view_name)
+
         cur.execute('SELECT indexname FROM pg_indexes WHERE indexname like %s', (existing_prefix + '%', ))
         existing_indexes = set()
         for row in cur:
@@ -189,7 +260,7 @@ class PostGISDB(object):
             if index_name.startswith(existing_prefix) and not index_name.startswith((new_prefix, backup_prefix)):
                 # check for overlapping prefixes: osm_ but not osm_new_ or osm_backup_
                 existing_indexes.add(index_name)
-        
+
         cur.execute('SELECT relname FROM pg_class WHERE relname like %s', (existing_prefix + '%_id_seq', ))
         existing_seq = set()
         for row in cur:
@@ -204,73 +275,100 @@ class PostGISDB(object):
             table_name = row[0]
             new_tables.append(table_name)
 
+        cur.execute('SELECT viewname FROM pg_views WHERE viewname like %s', (new_prefix + '%', ))
+        new_views = []
+        for row in cur:
+            view_name = row[0]
+            new_views.append(view_name)
+
         cur.execute('SELECT indexname FROM pg_indexes WHERE indexname like %s', (new_prefix + '%', ))
         new_indexes = set()
         for row in cur:
             index_name = row[0]
             new_indexes.add(index_name)
-        
+
         cur.execute('SELECT relname FROM pg_class WHERE relname like %s', (new_prefix + '%_id_seq', ))
         new_seq = []
         for row in cur:
             seq_name = row[0]
             new_seq.append(seq_name)
 
-        
+
         if not new_tables:
             raise RuntimeError('did not found tables to swap')
-        
+
         # rename existing tables (osm_) to backup_prefix (osm_backup_)
         for table_name in existing_tables:
             rename_to = table_name.replace(existing_prefix, backup_prefix)
             cur.execute('ALTER TABLE "%s" RENAME TO "%s"' % (table_name, rename_to))
 
             for idx in existing_indexes:
-                if idx in (table_name + '_geom', table_name + '_pkey') or idx.startswith(table_name + '_trgm_idx_'):
+                if idx in (table_name + '_geom', table_name + '_pkey') or idx.startswith(table_name + '_trgm_idx_') or idx.startswith(table_name + '_idx_'):
                     new_idx = idx.replace(table_name, rename_to, 1)
                     cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (idx, new_idx))
             if table_name + '_id_seq' in existing_seq:
                 cur.execute('ALTER SEQUENCE "%s" RENAME TO "%s"' % (table_name + '_id_seq', rename_to + '_id_seq'))
-            cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, table_name))
-            
+            if self.use_geometry_columns_table:
+                cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, table_name))
+
+        # rename existing views (osm_) to backup_prefix (osm_backup_)
+        for view_name in existing_views:
+            rename_to = view_name.replace(existing_prefix, backup_prefix)
+            cur.execute('ALTER VIEW "%s" RENAME TO "%s"' % (view_name, rename_to))
+
+            if self.use_geometry_columns_table:
+                cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, view_name))
+
         # rename new tables (osm_new_) to existing_prefix (osm_)
         for table_name in new_tables:
             rename_to = table_name.replace(new_prefix, existing_prefix)
             cur.execute('ALTER TABLE "%s" RENAME TO "%s"' % (table_name, rename_to))
 
             for idx in new_indexes:
-                if idx in (table_name + '_geom', table_name + '_pkey') or idx.startswith(table_name + '_trgm_idx_'):
+                if idx in (table_name + '_geom', table_name + '_pkey') or idx.startswith(table_name + '_trgm_idx_') or idx.startswith(table_name + '_idx_'):
                     new_idx = idx.replace(table_name, rename_to, 1)
                     cur.execute('ALTER INDEX "%s" RENAME TO "%s"' % (idx, new_idx))
             if table_name + '_id_seq' in new_seq:
                 cur.execute('ALTER SEQUENCE "%s" RENAME TO "%s"' % (table_name + '_id_seq', rename_to + '_id_seq'))
-            cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, table_name))
-        
+            if self.use_geometry_columns_table:
+                cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, table_name))
+
+        # rename new views (osm_new_) to existing_prefix (osm_)
+        for view_name in new_views:
+            rename_to = view_name.replace(new_prefix, existing_prefix)
+            cur.execute('ALTER VIEW "%s" RENAME TO "%s"' % (view_name, rename_to))
+
+            if self.use_geometry_columns_table:
+                cur.execute('UPDATE geometry_columns SET f_table_name = %s WHERE f_table_name = %s', (rename_to, view_name))
+
+
     def remove_tables(self, prefix):
         cur = self.connection.cursor()
         cur.execute('SELECT tablename FROM pg_tables WHERE tablename like %s', (prefix + '%', ))
         remove_tables = [row[0] for row in cur]
-        
+
         for table_name in remove_tables:
             cur.execute("DROP TABLE %s CASCADE" % (table_name, ))
-            cur.execute("DELETE FROM geometry_columns WHERE f_table_name = %s", (table_name, ))
-        
+            if self.use_geometry_columns_table:
+                cur.execute("DELETE FROM geometry_columns WHERE f_table_name = %s", (table_name, ))
+
 
     def remove_views(self, prefix):
         cur = self.connection.cursor()
         cur.execute('SELECT viewname FROM pg_views WHERE viewname like %s', (prefix + '%', ))
         remove_views = [row[0] for row in cur]
-        
+
         for view_name in remove_views:
             cur.execute('DROP VIEW "%s" CASCADE' % (view_name, ))
-            cur.execute("DELETE FROM geometry_columns WHERE f_table_name = %s", (view_name, ))
-        
-    
+            if self.use_geometry_columns_table:
+                cur.execute("DELETE FROM geometry_columns WHERE f_table_name = %s", (view_name, ))
+
+
     def create_views(self, mappings, ignore_errors=False):
         for mapping in mappings.values():
             if isinstance(mapping, UnionView):
                 PostGISUnionView(self, mapping).create(ignore_errors=ignore_errors)
-    
+
     def create_generalized_tables(self, mappings):
         mappings = [m for m in mappings.values() if isinstance(m, GeneralizedTable)]
         for mapping in sorted(mappings, key=lambda x: x.name, reverse=True):
@@ -298,7 +396,6 @@ class PostGISDB(object):
         cur.execute("VACUUM ANALYZE")
         self.connection.set_isolation_level(old_isolation_level)
 
-
 class PostGISUnionView(object):
     def __init__(self, db, mapping):
         self.mapping = mapping
@@ -322,10 +419,11 @@ class PostGISUnionView(object):
         selects = '\nUNION ALL\n'.join(selects)
 
         stmt = 'CREATE VIEW "%s" as (\n%s\n)' % (self.view_name, selects)
-        
+
         return stmt
 
     def _geom_table_stmt(self):
+        assert self.db.use_geometry_columns_table
         stmt = "insert into geometry_columns values ('', 'public', '%s', 'geometry', 2, %d, 'GEOMETRY')" % (
             self.view_name, self.db.srid)
         return stmt
@@ -349,22 +447,18 @@ class PostGISUnionView(object):
     def create(self, ignore_errors):
         cur = self.db.connection.cursor()
         cur.execute('BEGIN')
-        try:
-            cur.execute('SAVEPOINT pre_create_view')
-            cur.execute('SELECT * FROM pg_views WHERE viewname = %s', (self.view_name, ))
-            if cur.fetchall():
-                cur.execute('DROP VIEW %s' % (self.view_name, ))
-            cur.execute(self._view_stmt())
-        except psycopg2.ProgrammingError:
-            cur.execute('ROLLBACK TO SAVEPOINT pre_create_view')
-            if not ignore_errors:
-                raise
 
-        cur.execute('SELECT * FROM geometry_columns WHERE f_table_name = %s', (self.view_name, ))
-        if cur.fetchall():
-            # drop old entry to handle changes of SRID
-            cur.execute('DELETE FROM geometry_columns WHERE f_table_name = %s', (self.view_name, ))
-        cur.execute(self._geom_table_stmt())
+        self.db.drop_table_or_view(cur, self.view_name)
+
+        with self.db.savepoint(cur, raise_errors=not ignore_errors):
+            cur.execute(self._view_stmt())
+
+        if self.db.use_geometry_columns_table:
+            cur.execute('SELECT * FROM geometry_columns WHERE f_table_name = %s', (self.view_name, ))
+            if cur.fetchall():
+                # drop old entry to handle changes of SRID
+                cur.execute('DELETE FROM geometry_columns WHERE f_table_name = %s', (self.view_name, ))
+            cur.execute(self._geom_table_stmt())
 
 
 class PostGISGeneralizedTable(object):
@@ -373,11 +467,8 @@ class PostGISGeneralizedTable(object):
         self.mapping = mapping
         self.table_name = db.to_tablename(mapping.name)
 
-    def _idx_stmt(self):
-        return 'CREATE INDEX "%s_geom" ON "%s" USING GIST (geometry)' % (
-            self.table_name, self.table_name)
-
     def _geom_table_stmt(self):
+        assert self.db.use_geometry_columns_table
         stmt = "insert into geometry_columns values ('', 'public', '%s', 'geometry', 2, %d, 'GEOMETRY')" % (
             self.table_name, self.db.srid)
         return stmt
@@ -386,18 +477,17 @@ class PostGISGeneralizedTable(object):
         fields = ', '.join([n for n, t in self.mapping.fields])
         if fields:
             fields += ','
+        where = ' WHERE ST_IsValid(ST_SimplifyPreserveTopology(geometry, %f))' % (self.mapping.tolerance,)
         if self.mapping.where:
-            where = ' WHERE ' + self.mapping.where
-        else:
-            where = ''
-        
+            where = ' AND '.join([where, self.mapping.where])
+
         if config.imposm_pg_serial_id:
             serial_column = "id, "
         else:
             serial_column = ""
-    
+
         return """CREATE TABLE "%s" AS (SELECT %s osm_id, %s
-            ST_Simplify(geometry, %f) as geometry from "%s"%s)""" % (
+            ST_SimplifyPreserveTopology(geometry, %f) as geometry from "%s"%s)""" % (
             self.table_name, serial_column, fields, self.mapping.tolerance,
             self.db.to_tablename(self.mapping.origin.name),
             where)
@@ -405,20 +495,23 @@ class PostGISGeneralizedTable(object):
     def create(self):
         cur = self.db.connection.cursor()
         cur.execute('BEGIN')
-        try:
-            cur.execute('SAVEPOINT pre_drop_table')
-            cur.execute('DROP TABLE "%s" CASCADE' % (self.table_name, ))
-        except psycopg2.ProgrammingError:
-            cur.execute('ROLLBACK TO SAVEPOINT pre_drop_table')
-        
-        cur.execute(self._stmt())
-        cur.execute(self._idx_stmt())
 
-        cur.execute('SELECT * FROM geometry_columns WHERE f_table_name = %s', (self.table_name, ))
-        if cur.fetchall():
-            # drop old entry to handle changes of SRID
-            cur.execute('DELETE FROM geometry_columns WHERE f_table_name = %s', (self.table_name, ))
-        cur.execute(self._geom_table_stmt())
+        self.db.drop_table_or_view(cur, self.table_name)
+
+        cur.execute(self._stmt())
+
+        if self.db.use_geometry_columns_table:
+            cur.execute('SELECT * FROM geometry_columns WHERE f_table_name = %s', (self.table_name, ))
+            if cur.fetchall():
+                # drop old entry to handle changes of SRID
+                cur.execute('DELETE FROM geometry_columns WHERE f_table_name = %s', (self.table_name, ))
+            cur.execute(self._geom_table_stmt())
 
 class TrigramIndex(object):
     pass
+
+class StringIndex(object):
+    pass
+
+class Index(object):
+    pass
