@@ -37,7 +37,13 @@ from imposm.mapping import (
 # 
 # # skip relations with more rings (0 skip nothing)
 # imposm.config.imposm_multipolygon_max_ring = 0
-
+# 
+# # split ways that are longer than x nodes (0 to split nothing)
+# imposm.config.imposm_linestring_max_length = 50
+# 
+# # cache coords in a compact storage (with delta encoding)
+# # use this when memory is limited (default)
+# imposm.config.imposm_compact_coords_cache = True
 
 # set_default_name_type(LocalizedName(['name:en', 'int_name', 'name']))
 
@@ -246,7 +252,7 @@ transport_areas = Polygons(
             'station',
         ),
         'aeroway': (
-            'aerodome',
+            'aerodrome',
             'terminal',
             'helipad',
             'apron',
@@ -467,4 +473,4 @@ roads_gen0 = UnionView(
         ('z_order', 0),
     ),
     mappings = [railways_gen0, mainroads_gen0, motorways_gen0],
-)
\ No newline at end of file
+)
