@@ -1,12 +1,12 @@
 # -:- encoding: UTF8 -:-
 # Copyright 2011 Omniscale (http://omniscale.com)
-# 
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -45,11 +45,11 @@ default_name_field = None
 def set_default_name_type(type, column_name='name'):
     """
     Set new default type for 'name' field.
-    
+
     ::
-        
+
         set_default_name_type(LocalizedName(['name:en', 'int_name', 'name']))
-    
+
     """
     global default_name_field
     default_name_field = column_name, type
@@ -90,11 +90,11 @@ class Mapping(object):
         self.name = name
         self.mapping = mapping
         self.fields = fields or tuple(self.fields)
+        self.limit_to_polygon = None
         if with_type_field is not None:
             # allow subclass to define other default by setting it as class variable
             self.with_type_field = with_type_field
-        if self.with_type_field:
-            self._add_type_field()
+        self._add_type_field()
         self._add_name_field()
         if field_filter:
             self.field_filter = field_filter
@@ -108,11 +108,18 @@ class Mapping(object):
                 self.fields = (default_name_field,) + self.fields
             else:
                 self.fields = (('name', Name()),) + self.fields
-        
+
     def _add_type_field(self):
         """
         Add type field.
         """
+        if not self.with_type_field:
+            return
+
+        for name, type_ in self.fields:
+            if name == 'type':
+                # do not add type field if already present
+                return
         self.fields = (('type', Type()), ) + self.fields
 
     @property
@@ -120,12 +127,12 @@ class Mapping(object):
         if not self._insert_stmt:
             self._insert_stmt = self.table('osm_' + self.name, self).insert_stmt
         return self._insert_stmt
-    
+
     def extra_field_names(self):
         extra_field_names = []
         for field_name, field_filter in self.field_filter:
             extra_field_names.append(field_name)
-        
+
         for field_name, field in self.fields:
             field_names = field.extra_fields()
             if field_names is not None:
@@ -133,27 +140,39 @@ class Mapping(object):
             else:
                 extra_field_names.append(field_name)
         return extra_field_names
-    
+
     def build_geom(self, osm_elem):
         try:
             geom = self.geom_builder.build_checked_geom(osm_elem)
+            if self.limit_to_polygon is not None:
+                geom = self.limit_to_polygon.intersection(geom)
             osm_elem.geom = geom
         except imposm.geom.InvalidGeometryError, ex:
             raise DropElem('invalid geometry: %s' % (ex, ))
-    
+        except imposm.geom.EmtpyGeometryError, ex:
+            raise DropElem(ex)
+
     def field_values(self, osm_elem):
         return [t.value(osm_elem.tags.get(n), osm_elem) for n, t in self.fields]
-    
+
+    def field_dict(self, osm_elem):
+        result = dict((n, t.value(osm_elem.tags.get(n), osm_elem)) for n, t in self.fields)
+        if self.with_type_field:
+            del result['type']
+        result[osm_elem.cls] = osm_elem.type
+        return result
+
     def filter(self, osm_elem):
         [t.filter(osm_elem.tags.get(n), osm_elem) for n, t in self.field_filter]
-    
+
     def __repr__(self):
         return '<Mapping for %s>' % self.name
 
 
 class TagMapper(object):
-    def __init__(self, mappings):
+    def __init__(self, mappings, limit_to=None):
         self.mappings = mappings
+        self.limit_to_polygon = limit_to
         self._init_map()
 
     def _init_map(self):
@@ -174,7 +193,7 @@ class TagMapper(object):
             elif mapping.table is PolygonTable:
                 tags = self.polygon_tags
                 add_to = self.polygon_mappings
-            
+
             for extra in mapping.extra_field_names():
                 tags.setdefault(extra, set()).add('__any__')
 
@@ -184,11 +203,14 @@ class TagMapper(object):
                     tags.setdefault(tag, set()).add(type)
                     add_to[tag].setdefault(type, []).append(mapping)
 
+            # add limit_to polygon to each mapping
+            mapping.limit_to_polygon = self.limit_to_polygon
+
     def for_nodes(self, tags):
         return self._mapping_for_tags(self.point_mappings, tags)
 
     def for_ways(self, tags):
-        return (self._mapping_for_tags(self.line_mappings, tags) + 
+        return (self._mapping_for_tags(self.line_mappings, tags) +
                 self._mapping_for_tags(self.polygon_mappings, tags))
 
     def for_relations(self, tags):
@@ -218,7 +240,7 @@ class TagMapper(object):
         tags = dict()
         for k, v in self.line_tags.iteritems():
             tags.setdefault(k, set()).update(v)
-        
+
         for k, v in self.polygon_tags.iteritems():
             tags.setdefault(k, set()).update(v)
         return self._tag_filter(tags)
@@ -272,7 +294,7 @@ class TagMapper(object):
                         new_mappings.append(proc)
                 if new_mappings:
                     result.append(((tag_name, tag_value), tuple(new_mappings)))
-        
+
         return result
 
 
@@ -287,7 +309,7 @@ class PolygonTable(object):
 class Points(Mapping):
     """
     Table class for point features.
-    
+
     :PostGIS datatype: POINT
     """
     table = PointTable
@@ -297,7 +319,7 @@ class Points(Mapping):
 class LineStrings(Mapping):
     """
     Table class for line string features.
-    
+
     :PostGIS datatype: LINESTRING
     """
     table = LineStringTable
@@ -307,7 +329,7 @@ class LineStrings(Mapping):
 class Polygons(Mapping):
     """
     Table class for polygon features.
-    
+
     :PostGIS datatype: GEOMETRY (POLYGON does not support multi-polygons)
     """
     table = PolygonTable
@@ -350,7 +372,7 @@ class UnionView(object):
         if 'type' not in self.fields and any(m.with_type_field for m in self.mappings):
             self.fields += (('type', None), )
 
-        
+
 class DropElem(Exception):
     pass
 
@@ -360,12 +382,12 @@ class FieldType(object):
         """
         List with names of fields (keys) that should be processed
         during read-phase.
-        
+
         Return ``None`` to use the field name from the mapping.
         Return ``[]`` if no extra fields (keys) are required.
         """
         return None
-    
+
     def value(self, val, osm_elem):
         return val
 
@@ -393,7 +415,7 @@ class Type(FieldType):
                 ('road_class', Type(),),
             ),
         )
-    
+
     :PostgreSQL datatype: VARCHAR(255)
 
     .. versionadded:: 2.4.0
@@ -411,11 +433,11 @@ class Class(FieldType):
     """
     Field for class values (i.e. the *key* of the mapped key/value).
 
-    Use this if you want to store the key that was used for 
+    Use this if you want to store the key that was used for
     this mapping. For example, the following mapping will
     create a column ``class`` that will have the value
     ``landuse`` or ``natural``, depending on the feature.
-    
+
     ::
 
         landusages = Polygons(
@@ -434,7 +456,7 @@ class Class(FieldType):
         )
 
     :PostgreSQL datatype: VARCHAR(255)
-    
+
     .. versionadded:: 2.4.0
 
     """
@@ -449,7 +471,7 @@ class Class(FieldType):
 class String(FieldType):
     """
     Field for string values.
-    
+
     :PostgreSQL datatype: VARCHAR(255)
     """
     column_type = "VARCHAR(255)"
@@ -457,14 +479,14 @@ class String(FieldType):
 class Name(String):
     """
     Field for name values.
-    
+
     Filters out common FixMe values.
-    
+
     :PostgreSQL datatype: VARCHAR(255)
 
     .. versionadded:: 2.3.0
     """
-    
+
     filter_out_names = set((
         'fixme', 'fix me', 'fix-me!',
         '0', 'none', 'n/a', 's/n',
@@ -486,18 +508,18 @@ class LocalizedName(Name):
     Field for localized name values.
     Checks different name keys and uses the first key with
     a valid value.
-    
+
     :param coalesce: list of name keys to check
     :PostgreSQL datatype: VARCHAR(255)
-    
+
     .. versionadded:: 2.3.0
     """
     def __init__(self, coalesce=['name', 'int_name']):
         self.coalesce_keys = coalesce
-    
+
     def extra_fields(self):
         return self.coalesce_keys
-    
+
     def value(self, val, osm_elem):
         for key in self.coalesce_keys:
             val = osm_elem.tags.get(key)
@@ -512,7 +534,7 @@ class Bool(FieldType):
     """
     Field for boolean values.
     Converts false, no, 0 to False and true, yes, 1 to True.
-    
+
     :PostgreSQL datatype: SMALLINT
     """
     # there was a reason this is not BOOL
@@ -543,7 +565,7 @@ class Direction(FieldType):
     Converts `yes`, `true` and `1` to ``1`` for one ways in the direction of
     the way, `-1` to ``-1`` for one ways against the direction of the way and
     ``0`` for all other values.
-    
+
     :PostgreSQL datatype: SMALLINT
     """
     column_type = "SMALLINT"
@@ -559,36 +581,36 @@ class Direction(FieldType):
 class PseudoArea(FieldType):
     """
     Field for the (pseudo) area of a polygon in square meters.
-    
+
     The value is just an approximation since the geometries are in
     EPSG:4326 and not in a equal-area projection. The approximation
     is good for smaller polygons (<1%) and should be precise enough
     to compare geometries for rendering order (larger below smaller).
-    
+
     The area of the geometry is multiplied by the cosine of
     the mid-latitude to compensate the reduced size towards
     the poles.
-    
+
     :PostgreSQL datatype: REAL
-    
+
     .. versionadded:: 2.3.0
     """
-    
+
     column_type = "REAL"
-    
+
     def value(self, val, osm_elem):
         area = osm_elem.geom.area
         if not area:
             return None
-        
+
         extent = osm_elem.geom.bounds
         mid_lat = extent[1] + (abs(extent[3] - extent[1]) / 2)
         sqr_deg = area * math.cos(math.radians(mid_lat))
-        
+
         # convert deg^2 to m^2
         sqr_m = (math.sqrt(sqr_deg) * (40075160 / 360))**2
         return sqr_m
-    
+
     def extra_fields(self):
         return []
 
@@ -597,7 +619,7 @@ class OneOfInt(FieldType):
     Field type for integer values.
     Converts values to integers, drops element if is not included in
     ``values``.
-    
+
     :PostgreSQL datatype: SMALLINT
     """
     column_type = "SMALLINT"
@@ -614,7 +636,7 @@ class Integer(FieldType):
     """
     Field type for integer values.
     Converts values to integers, defaults to ``NULL``.
-    
+
     :PostgreSQL datatype: INTEGER
     """
     column_type = "INTEGER"
@@ -628,12 +650,12 @@ class Integer(FieldType):
 class ZOrder(FieldType):
     """
     Field type for z-ordering based on the feature type.
-    
+
     :param types: list of mapped feature types,
         from highest to lowest ranking
     :PostgreSQL datatype: SMALLINT
     """
-    
+
     column_type = "SMALLINT"
 
     def __init__(self, types):
@@ -651,12 +673,12 @@ class ZOrder(FieldType):
 class WayZOrder(FieldType):
     """
     Field type for z-ordered based on highway types.
-    
+
     Ordering based on the osm2pgsql z-ordering:
     From ``roads`` = 3 to ``motorways`` = 9, ``railway`` = 7 and unknown = 0.
     Ordering changes with ``tunnels`` by -10, ``bridges`` by +10 and
     ``layer`` by 10 * ``layer``.
-    
+
     :PostgreSQL datatype: SMALLINT
     """
 
@@ -680,7 +702,7 @@ class WayZOrder(FieldType):
     }
 
     brunnel_bool = Bool()
-    
+
     def extra_fields(self):
         return []
 
